@@ -36,9 +36,31 @@ import shutil
 import locale
 import datetime
 
-from weyland import Lexer, RECOGNIZED_LANGUAGES, LANGUAGES
 from logging import info, warning, error
 
+# Ugly hack to have the dev version of Weyland on my computer instead of the one loaded through pypi
+could_be = [r"C:\Users\damie_000\Documents\GitHub\tallentaa\projets\weyland\weyland\__init__.py",
+            '/home/damien/Documents/tallentaa/projets/weyland/weyland/__init__.py']
+location = None
+for cb in could_be:
+    if os.path.exists(cb):
+        location = cb
+        break
+if location is not None:
+    import sys
+    import importlib.util
+    spec = importlib.util.spec_from_file_location("weyland", location)
+    weyland = importlib.util.module_from_spec(spec)
+    sys.modules[spec.name] = weyland
+    spec.loader.exec_module(weyland)
+    Lexer = weyland.Lexer
+    RECOGNIZED_LANGUAGES = weyland.RECOGNIZED_LANGUAGES
+    LANGUAGES = weyland.LANGUAGES
+    __version__ = weyland.__version__
+else:
+    from weyland import Lexer, RECOGNIZED_LANGUAGES, LANGUAGES, __version__
+print('Using Weyland version:', __version__)
+
 #-------------------------------------------------------------------------------
 # Constants and globals
 #-------------------------------------------------------------------------------
@@ -79,19 +101,38 @@ class Generation:
         try:
             dt = datetime.datetime.now()
             found = False
-            # first try, our lang, our encoding
-            for lang, encoding in locale.locale_alias.items():
-                if lang.startswith(default_lang) and default_encoding in encoding.lower():
-                    try:
-                        locale.setlocale(locale.LC_TIME, (lang, encoding))
-                        info(f'Locale set to {lang} and encoding {encoding}')
-                        found = True
-                        break
-                    except locale.Error:
-                        if (error_locale):
-                            warning(f'Impossible to set locale to ({lang}, {encoding}).')
-                        pass
-            # second try, our lang, any encoding
+            # first try
+            language_alias = {'en': ['en_US'], 'fr': ['fr_FR']}
+            encoding_alias = {'utf-8': ['utf8'], 'utf8': ['utf-8']}
+            all_alias = [(default_lang, default_encoding)]
+            if default_lang in language_alias:
+                for elem in language_alias[default_lang]:
+                    all_alias.append((elem, default_encoding))
+                    if default_encoding in encoding_alias:
+                        for enco in encoding_alias[default_encoding]:
+                            all_alias.append((elem, enco))
+            for lg, en in all_alias:
+                try:
+                    locale.setlocale(locale.LC_TIME, (lg, en))
+                    info(f'Locale set to {lg} and encoding {en}')
+                    found = True
+                    break
+                except locale.Error:
+                    pass
+            # second try, our lang, our encoding
+            if not found:
+                for lang, encoding in locale.locale_alias.items():
+                    if lang.startswith(default_lang) and default_encoding in encoding.lower():
+                        try:
+                            locale.setlocale(locale.LC_TIME, (lang, encoding))
+                            info(f'Locale set to {lang} and encoding {encoding}')
+                            found = True
+                            break
+                        except locale.Error:
+                            if (error_locale):
+                                warning(f'Impossible to set locale to ({lang}, {encoding}).')
+                            pass
+            # third try, our lang, any encoding
             if not found:
                 for lang, encoding in locale.locale_alias.items():
                     if lang.startswith(default_lang):
@@ -230,7 +271,7 @@ def multi_find(string, finds):
 
 
 def find_unescaped(line, motif, start=0):
-    "Used by super_strip and code handling and multi_find"
+    "Used by super_strip, code handling and multi_find"
     index = start
     while index < len(line):
         found = line.find(motif, index)
@@ -239,7 +280,7 @@ def find_unescaped(line, motif, start=0):
         elif found == 0 or line[found - 1] != '\\':
             return found
         else:
-            index = found + len(motif)
+            index = found + 1 #len(motif)
     return -1
 
 
@@ -318,26 +359,9 @@ def make_id(string):
 
 
 def write_code(line, code_lang):
-    tokens = Lexer(LANGUAGES[code_lang]['tokens']).lex(line)
-    tokens_by_index = {}
-    next_stop = None
-    string = ''
-    for tok in tokens:
-        tokens_by_index[tok.first] = tok
-    for index_char, char in enumerate(line):
-        if index_char in tokens_by_index:
-            tok = tokens_by_index[index_char]
-            string += f'<span class="{tok.typ}">{char}'
-            next_stop = tok.last
-            if next_stop == index_char:
-                string += f'</span>'
-                next_stop = None
-        elif next_stop is not None and index_char == next_stop:
-            string += f'{char}</span>'
-            next_stop = None
-        else:
-            string += safe(char)
-    return string
+    line = line.replace('\\@', '@') # warning bug if \\@
+    #print('write_code', code_lang, line)
+    return Lexer(LANGUAGES[code_lang], debug=False).to_html(text=line)
 
 
 def check_link(link, links, inner_links):
@@ -374,54 +398,55 @@ def process_string(line, gen=None):
         char = line[char_index]
         prev_char, next_char, prev_prev_char = prev_next(line, char_index)
         # Paragraph et span class (div class are handled elsewhere)
-        if char == '{' and next_char == '{' and prev_char != '\\':
+        if char == '{' and next_char == '{' and prev_char != '\\' and line.find('}}', char_index+1) != -1:
             continue
         if char == '{' and prev_char == '{' and prev_prev_char != '\\':
             ending = line.find('}}', char_index)
-            inside = line[char_index + 1:ending]
-            cls = ''
-            ids = ''
-            txt = ''
-            state = 'start'
-            for c in inside:
-                if state == 'start':
-                    if c == '.':
-                        state = 'cls'
-                        cls += c
-                    elif c == '#':
-                        state = 'ids'
-                        ids += c
-                    else:
-                        state = 'txt'
+            if ending != -1:
+                inside = line[char_index + 1:ending]
+                cls = ''
+                ids = ''
+                txt = ''
+                state = 'start'
+                for c in inside:
+                    if state == 'start':
+                        if c == '.':
+                            state = 'cls'
+                            cls += c
+                        elif c == '#':
+                            state = 'ids'
+                            ids += c
+                        else:
+                            state = 'txt'
+                            txt += c
+                    elif state == 'cls':
+                        if c != ' ':
+                            cls += c
+                        else:
+                            state = 'start'
+                    elif state == 'ids':
+                        if c != ' ':
+                            ids += c
+                        else:
+                            state = 'start'
+                    elif state == 'txt': # you can't escape txt mode
                         txt += c
-                elif state == 'cls':
-                    if c != ' ':
-                        cls += c
+                if len(txt) > 0:
+                    if len(ids) > 0 and len(cls) > 0:
+                        new_line += f'<span id="{ids[1:]}" class="{cls[1:]}">{txt}</span>'
+                    elif len(ids) > 0:
+                        new_line += f'<span id="{ids[1:]}">{txt}</span>'
+                    elif len(cls) > 0:
+                        new_line += f'<span class="{cls[1:]}">{txt}</span>'
                     else:
-                        state = 'start'
-                elif state == 'ids':
-                    if c != ' ':
-                        ids += c
-                    else:
-                        state = 'start'
-                elif state == 'txt': # you can't escape txt mode
-                    txt += c
-            if len(txt) > 0:
-                if len(ids) > 0 and len(cls) > 0:
-                    new_line += f'<span id="{ids[1:]}" class="{cls[1:]}">{txt}</span>'
-                elif len(ids) > 0:
-                    new_line += f'<span id="{ids[1:]}">{txt}</span>'
-                elif len(cls) > 0:
-                    new_line += f'<span class="{cls[1:]}">{txt}</span>'
+                        new_line += f'<span>{txt}</span>'
                 else:
-                    new_line += f'<span>{txt}</span>'
-            else:
-                if len(ids) > 0:
-                    gen['NEXT_PAR_ID'] = ids[1:]
-                if len(cls) > 0:
-                    gen['NEXT_PAR_CLASS'] = cls[1:]
-            char_index = ending + 1
-            continue
+                    if len(ids) > 0:
+                        gen['NEXT_PAR_ID'] = ids[1:]
+                    if len(cls) > 0:
+                        gen['NEXT_PAR_CLASS'] = cls[1:]
+                char_index = ending + 1
+                continue
         # Links and images
         if char == '[' and prev_char != '\\' and next_char != '[': # [[ the first is not a link!
             ending = line.find(']', char_index)
@@ -534,7 +559,7 @@ def process_string(line, gen=None):
         if char == '@' and next_char == '@' and prev_char != '\\':
             continue
         if char == '@' and prev_char == '@' and prev_prev_char != '\\':
-            if next_char is not None and line.index("@@", char_index + 1) != -1:
+            if next_char is not None and line.find("@@", char_index + 1) != -1:
                 ending = find_unescaped(line, '@@', char_index)
                 code = line[char_index + 1:ending]
                 length = len(code) + 2
